@@ -262,7 +262,7 @@ class Builder {
         if (!rule.parts.length) continue
         let start = rule.parts[0]
         for (let t of start.terminal ? [start] : first[start.name] || [])
-          if (!startTokens.includes(t)) startTokens.push(t)
+          if (t && !startTokens.includes(t)) startTokens.push(t)
         if (start.terminal && rule.parts.length == 1 && !rules.some(r => r != rule && r.parts[0] == start))
           skip.push(start)
         else
@@ -435,7 +435,7 @@ class Builder {
       termNames
     } = this.prepareParser()
 
-    let mod = this.options.moduleStyle || "es", ts = this.options.typeScript ? ": any" : ""
+    let mod = this.options.moduleStyle || "es"
 
     let gen = "// This file was generated by lezer-generator. You probably shouldn't edit it.\n", head = gen
     let imports: {[source: string]: string[]} = {}, imported: {[spec: string]: string} = Object.create(null)
@@ -475,7 +475,7 @@ class Builder {
     })
 
     function specializationTableString(table: {[name: string]: number}) {
-      return "{__proto__:null," + Object.keys(table).map(key => `${/\W/.test(key) ? JSON.stringify(key) : key}:${table[key]}`)
+      return "{__proto__:null," + Object.keys(table).map(key => `${/^(\d+|[a-zA-Z_]\w*)$/.test(key) ? key : JSON.stringify(key)}:${table[key]}`)
         .join(", ") + "}"
     }
 
@@ -483,6 +483,7 @@ class Builder {
     let specialized = rawSpecialized.map(v => {
       if (v instanceof ExternalSpecializer) {
         let name = importName(v.ast.id.name, v.ast.source)
+        let ts = this.options.typeScript ? ": any" : ""
         return `{term: ${v.term!.id}, get: (value${ts}, stack${ts}) => (${name}(value, stack) << 1)${
           v.ast.type == "extend" ? ` | ${Specialize.Extend}` : ''}, external: ${name}${
           v.ast.type == "extend" ? ', extend: true' : ''}}`
@@ -490,6 +491,7 @@ class Builder {
         let tableName = getName("spec_" + v.token.name.replace(/\W/g, ""))
         defined[tableName] = true
         specHead += `const ${tableName} = ${specializationTableString(v.table)}\n`
+        let ts = this.options.typeScript ? `: keyof typeof ${tableName}` : ""
         return `{term: ${v.token.id}, get: (value${ts}) => ${tableName}[value] || -1}`
       }
     })
@@ -541,6 +543,8 @@ class Builder {
       if (KEYWORDS.includes(id)) for (let i = 1;; i++) {
         id = "_".repeat(i) + name
         if (!(id in this.termTable)) break
+      } else if (!/^[\w$]+$/.test(name)) {
+        continue
       }
       terms.push(`${id}${mod == "cjs" ? ":" : " ="} ${this.termTable[name]}`)
     }
@@ -879,7 +883,11 @@ class Builder {
 
     if (!inline) this.built.push(new BuiltRule(rule.id.name, args, name))
     this.currentSkip.push(skip)
-    this.defineRule(name, this.normalizeExpr(expr))
+    let parts = this.normalizeExpr(expr)
+    if (parts.length > 100 * (expr instanceof ChoiceExpression ? expr.exprs.length : 1))
+      this.warn(`Rule ${rule.id.name} is generating a lot (${parts.length}) of choices.\n  Consider splitting it up or reducing the amount of ? or | operator uses.`, rule.start)
+    if (/\brulesize\b/.test(verbose) && parts.length > 10) console.log(`Rule ${rule.id.name}: ${parts.length} variants`)
+    this.defineRule(name, parts)
     this.currentSkip.pop()
     if (group) this.definedGroups.push({name, group, rule})
     return name
@@ -973,7 +981,7 @@ class Builder {
 
   resolveSpecialization(expr: SpecializeExpression) {
     let type = expr.type
-    let {name, props, dialect} = this.nodeInfo(expr.props, "d")
+    let {name, props, dialect, exported} = this.nodeInfo(expr.props, "d")
     let terminal = this.normalizeExpr(expr.token)
     if (terminal.length != 1 || terminal[0].terms.length != 1 || !terminal[0].terms[0].terminal)
       this.raise(`The first argument to '${type}' must resolve to a token`, expr.token.start)
@@ -996,6 +1004,10 @@ class Builder {
         }
         table.push({value, term: token, type, dialect, name})
         this.tokenOrigins[token.name] = {spec: term}
+        if (name || exported) {
+          if (!name) token.preserve = true
+          this.namedTerms[exported || name!] = token
+        }
       } else {
         if (known.type != type)
           this.raise(`Conflicting specialization types for ${JSON.stringify(value)} of ${term.name} (${type} vs ${known.type})`,
@@ -1991,7 +2003,7 @@ export function buildParser(text: string, options: BuildOptions = {}): LRParser
   return parser
 }
 
-const KEYWORDS = ["break", "case", "catch", "continue", "debugger", "default", "do", "else", "finally",
+const KEYWORDS = ["await", "break", "case", "catch", "continue", "debugger", "default", "do", "else", "finally",
                   "for", "function", "if", "return", "switch", "throw", "try", "var", "while", "with",
                   "null", "true", "false", "instanceof", "typeof", "void", "delete", "new", "in", "this",
                   "const", "class", "extends", "export", "import", "super", "enum", "implements", "interface",
